/*
 * Copyright (C) 2009-2012 Eiichiro Uchiumi. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 *     
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.eiichiro.jazzmaster.aspectj.frontend;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.annotation.SuppressAjWarnings;

import org.eiichiro.jazzmaster.Application;
import org.eiichiro.jazzmaster.frontend.Frontend;
import org.eiichiro.jazzmaster.frontend.Injection;

/**
 * {@code AspectJInjection} is {@code Injection} extension implementation to 
 * inject Jazzmaster service into application frontend with 
 * <a href="http://www.eclipse.org/aspectj/">AspectJ 5</a>. With Jazzmaster 
 * built-in {@code Injection}, you need to implement the following code to get a 
 * service injected application frontend: 
 * <pre>
 * HelloFrontend frontend = Application.getFrontend(HelloFrontend.class);
 * </pre>
 * With {@code AspectJInjection}, you can get it with the following code: 
 * <pre>
 * HelloFrontend helloFrontend = new HelloFrontend();
 * </pre>
 * {@code AspectJInjection} is implemented as the aspect and weaven into 
 * initialization (constructor execution) of {@code @Frontend} annotated class. 
 * So, though you instantiate application frontend directly, you can get the 
 * frontend instance that service has been injected.
 * {@code AspectJInjection} does not only make your code simple, but also make 
 * easy to integrate Jazzmaster application frontend with third-party libraries. 
 * For example, in case of using Apache Struts {@code Action} as Jazzmaster 
 * application frontend. If you use Jazzmaster built-in {@code Injection}, you 
 * need to customize the code of Apache Struts instantiating an {@code Action} 
 * class. If you use {@code AspectJInjection}, the code of service injection has 
 * been weaven into {@code Action} constructor. Therefore, you can use Apache 
 * Struts {@code Action} as Jazzmaster application frontend with NO Struts code 
 * enhancement.
 * 
 * @see Injection
 * @see Frontend
 * @see Application
 * @author <a href="mailto:eiichiro@eiichiro.org">Eiichiro Uchiumi</a>
 */
@Aspect
public class AspectJInjection extends Injection {

	/**
	 * The pointcut for weaving {@code Injection} into {@code @Frontend} 
	 * annotated class's constructor.
	 */
	@Pointcut("initialization((@(org.eiichiro.jazzmaster.frontend.Frontend) *+).new(..))")
	public void frontendInitialization() {}
	
	/**
	 * The advice which injects Jazzmaster service instance (proxy) into the 
	 * specified application frontend instance.
	 * 
	 * @param joinPoint The join point that represents {@code @Frontend} 
	 * annotated class's constructor execution.
	 */
	@SuppressAjWarnings("adviceDidNotMatch")
	@Before("frontendInitialization()")
	public void adviceFrontendInitialization(JoinPoint joinPoint) {
		Object frontend = joinPoint.getThis();
		inject(frontend);
	}
	
}
